"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const fs_1 = require("fs");
const path_1 = require("path");
const from_dir_1 = require("./from-dir");
const string_builder_1 = require("./string-builder");
function buildSandboxes(srcPath, chunk) {
    const chunkMode = chunk ? 'lazy' : 'eager';
    const home = path_1.resolve(srcPath);
    const sandboxes = findSandboxes(home);
    const filePath = path_1.resolve(__dirname, '../../build/shared/sandboxes.js');
    const fileContent = buildSandboxFileContents(sandboxes, home, chunkMode);
    // TODO: Remove next release post 3.1.0
    deleteDeprecatedSandboxFileIfNecessary(home);
    return new Promise((resolve, reject) => {
        fs_1.writeFile(filePath, fileContent, err => {
            if (err) {
                console.log(chalk_1.default.red('Unable to compile sandboxes.\n'));
                throw new Error(err.message);
            }
            console.log('Successfully compiled sandbox files.');
            resolve(filePath);
        });
    });
}
exports.buildSandboxes = buildSandboxes;
function findSandboxes(home) {
    const sandboxes = [];
    from_dir_1.fromDir(home, /\.sandbox.ts$/, (filename) => {
        let sandboxPath = filename.replace(home, '.').replace(/.ts$/, '').replace(/\\/g, '/');
        const contents = fs_1.readFileSync(filename, 'utf8');
        const matchSandboxOf = /\s?sandboxOf\s*\(\s*([^)]+?)\s*\)/g.exec(contents);
        if (matchSandboxOf) {
            const typeName = matchSandboxOf[1].split(',')[0].trim();
            const labelText = /label\s*:\s*['"](.+)['"]/g.exec(matchSandboxOf[0]);
            let scenarioMenuItems = [];
            const scenarioRegex = /\.add\s*\(['"](.+)['"]\s*,\s*{/g;
            let scenarioMatches;
            let scenarioIndex = 1;
            while ((scenarioMatches = scenarioRegex.exec(contents)) !== null) {
                scenarioMenuItems.push({ key: scenarioIndex, description: scenarioMatches[1] });
                scenarioIndex++;
            }
            let label = labelText ? labelText[1] : '';
            sandboxes.push({
                key: sandboxPath,
                searchKey: `${typeName}${label}`,
                name: typeName,
                label: label,
                scenarioMenuItems
            });
        }
    });
    return sandboxes;
}
function buildSandboxFileContents(sandboxes, home, chunkMode) {
    const content = new string_builder_1.StringBuilder();
    content.addLine(`function getSandboxMenuItems() {`);
    content.addLine(`return ${JSON.stringify(sandboxes)};`);
    content.addLine(`}`);
    content.addLine('exports.getSandboxMenuItems = getSandboxMenuItems;');
    content.addLine(`function getSandbox(path) {`);
    content.addLine(`switch(path) {`);
    sandboxes.forEach(({ key }, i) => {
        let fullPath = path_1.join(home, key);
        // Normalize slash syntax for Windows/Unix filepaths
        fullPath = slash(fullPath);
        content.addLine(`case '${key}':`);
        content.addLine(`  return import( /* webpackMode: "${chunkMode}" */ '${fullPath}').then(_ => _.default.serialize('${key}'));`);
    });
    content.addLine(`}`);
    content.addLine(`}`);
    content.addLine('exports.getSandbox = getSandbox;');
    return content.dump();
}
// TODO: Remove
// Provided for minor release post sandboxes.ts resolution changes
function deleteDeprecatedSandboxFileIfNecessary(home) {
    const sandboxesFile = path_1.resolve(home, './sandboxes.ts');
    if (fs_1.existsSync(sandboxesFile)) {
        fs_1.unlinkSync(sandboxesFile);
    }
}
// Turns windows URL string ('c:\\etc\\') into URL node expects ('c:/etc/')
// https://github.com/sindresorhus/slash
function slash(input) {
    const isExtendedLengthPath = /^\\\\\?\\/.test(input);
    const hasNonAscii = /[^\u0000-\u0080]+/.test(input);
    if (isExtendedLengthPath || hasNonAscii) {
        return input;
    }
    return input.replace(/\\/g, '/');
}
